// Dimensions and margins
const heatmapMargin = {
    top: 50,
    right: 30,
    bottom: 100,
    left: 150,
  },
  heatmapWidth =
    960 - heatmapMargin.left - heatmapMargin.right,
  heatmapHeight =
    250 - heatmapMargin.top - heatmapMargin.bottom;

// Appending svg object
const heatmapSvg = d3
  .select('#heatmap')
  .append('svg')
  .attr(
    'width',
    heatmapWidth + heatmapMargin.left + heatmapMargin.right,
  )
  .attr(
    'height',
    heatmapHeight +
      heatmapMargin.top +
      heatmapMargin.bottom,
  )
  .append('g')
  .attr(
    'transform',
    `translate(${heatmapMargin.left},${heatmapMargin.top})`,
  );

// Read the csv file
d3.csv(
  'https://gist.githubusercontent.com/riyamathur1/9e6a69695dce47b51253ae50fdf87783/raw/637686002b0c59b5320a8506e4c4b199abde1581/atlanticstorms.csv',
).then((data) => {
  // Converting string -> numbers for all the data
  data.forEach((d) => {
    d.frequency = +d.frequency;
    d.year = +d.year;
  });

  // Grouping the data - one group per year and per name
  const dataGroup = Array.from(
    d3.group(
      data,
      (d) => d.year,
      (d) => d.name,
    ),
    ([key, value]) => ({ key, value }),
  );

  // X axis
  const x = d3
    .scaleBand()
    .range([0, heatmapWidth])
    .domain(data.map((d) => d.year))
    .padding(0.05);
  heatmapSvg
    .append('g')
    .attr('transform', `translate(0, ${heatmapHeight})`)
    .call(d3.axisBottom(x).tickSize(0))
    .select('.domain')
    .remove();

  // Y axis
  const y = d3
    .scaleBand()
    .range([heatmapHeight, 0])
    .domain(data.map((d) => d.name))
    .padding(0.05);
  heatmapSvg
    .append('g')
    .call(d3.axisLeft(y).tickSize(0))
    .select('.domain')
    .remove();

  // Building color scale
  const myColor = d3
    .scaleSequential()
    .interpolator(d3.interpolateInferno)
    .domain([1, d3.max(data, (d) => d.frequency)]);

  // Tooltip
  const tooltip = d3
    .select('#heatmap')
    .append('div')
    .style('opacity', 0)
    .attr('class', 'tooltip')
    .style('background-color', 'white')
    .style('border', 'solid')
    .style('border-width', '2px')
    .style('border-radius', '5px')
    .style('padding', '5px');

  // Three functs that change the tooltip when user hovers
  const mouseover = function (event, d) {
    tooltip.style('opacity', 1);
    d3.select(this)
      .style('stroke', 'black')
      .style('opacity', 1);
  };
  const mousemove = function (event, d) {
    tooltip
      .html('Frequency: ' + d.frequency)
      .style('left', event.x + 'px')
      .style('top', event.y + 28 + 'px');
  };
  const mouseleave = function (event, d) {
    tooltip.style('opacity', 0);
    d3.select(this)
      .style('stroke', 'none')
      .style('opacity', 0.8);
  };

  // adding squares
  heatmapSvg
    .selectAll()
    .data(data, function (d) {
      return d.year + ':' + d.name;
    })
    .join('rect')
    .attr('x', (d) => x(d.year))
    .attr('y', (d) => y(d.name))
    .attr('width', 30)
    .attr('height', 30)
    .style('fill', (d) => myColor(d.frequency))
    .style('stroke-width', 4)
    .style('stroke', 'none')
    .style('opacity', 0.8)
    .on('mouseover', mouseover)
    .on('mousemove', mousemove)
    .on('mouseleave', mouseleave);

  // Title for heatmap
  heatmapSvg
    .append('text')
    .attr('class', 'heatmap-title')
    .attr('x', heatmapWidth / 2)
    .attr('y', -20)
    .attr('text-anchor', 'middle')
    .style('font-size', '20px')
    .text('Storm Frequency');

  // X axis label
  heatmapSvg
    .append('text')
    .attr('text-anchor', 'end')
    .attr('x', heatmapWidth / 2)
    .attr('y', heatmapHeight + heatmapMargin.top)
    .text('Year');

  // Y axis label
  heatmapSvg
    .append('text')
    .attr('text-anchor', 'end')
    .attr('transform', 'rotate(-90)')
    .attr('y', -heatmapMargin.left + 20)
    .attr('x', -heatmapMargin.top - heatmapHeight / 2 + 80)
    .text('Storm Type');

  // Define the gradient
  const gradient = heatmapSvg
    .append('defs')
    .append('linearGradient')
    .attr('id', 'gradient')
    .attr('x1', '0%')
    .attr('x2', '100%')
    .attr('y1', '0%')
    .attr('y2', '0%');

  // Defining the gradient colors
  const colorRange = myColor.ticks(10); // Using 10 ticks for the range
  colorRange.forEach((d, i) => {
    gradient
      .append('stop')
      .attr('offset', `${(i / colorRange.length) * 100}%`)
      .attr('stop-color', myColor(d));
  });

  // Legend rectangle filled with the gradient
  const legendWidth = 150;
  const legendHeight = 20;
  heatmapSvg
    .append('rect')
    .attr('x', heatmapWidth - legendWidth) // Align to the right
    .attr('y', heatmapHeight + heatmapMargin.bottom - 40) // Positioned below the chart
    .attr('width', legendWidth)
    .attr('height', legendHeight)
    .style('fill', 'url(#gradient)');

  // Legend labels
  const legendScale = d3
    .scaleLinear()
    .domain([1, d3.max(data, (d) => d.frequency)])
    .range([0, legendWidth]);

  heatmapSvg
    .append('g')
    .attr('class', 'legend-axis')
    .attr(
      'transform',
      `translate(${heatmapWidth - legendWidth}, ${heatmapHeight + heatmapMargin.bottom - 20})`,
    ) // Position below the legend rectangle
    .call(d3.axisBottom(legendScale).ticks(5))
    .select('.domain')
    .remove();
});
